{ ============================================================================= }
{                          UNIT uAcropolis                                      }
{                                                                               }
{ Description: Demonstrates the ability of GLScene to load BSP files.           }
{ This demo includes collision detection.                                       }
{                                                                               }
{ Created June 2004 Skinhat                                                     }
{                                                                               }
{ ============================================================================= }
unit uAcropolis;

interface

uses
  Windows, GLCadencer, GLVectorFileObjects, GLScene, GLObjects, GLMisc,
  StdCtrls, Buttons, Controls, ExtCtrls, ComCtrls, Classes, Forms,
  GLWin32Viewer, GLFileMD2, GLGeomObjects,GLFileQ3BSP, GLTexture,dialogs,
  GLCollision,tga;

type
  TForm1 = class(TForm)
    GLScene1: TGLScene;
    GLCamera1: TGLCamera;
    GLLightSource1: TGLLightSource;
    GLSceneViewer1: TGLSceneViewer;
    GLCadencer1: TGLCadencer;
    GLFreeForm1: TGLFreeForm;
    GLMaterialLibrary1: TGLMaterialLibrary;
    sky: TGLPlane;
    sea: TGLPlane;
    GLSprite1: TGLSprite;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure GLCadencer1Progress(Sender: TObject; const deltaTime,
      newTime: Double);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private


  public

  end;

var
  Form1: TForm1;
  i: integer;

implementation

{$R *.DFM}

uses VectorGeometry, SysUtils, Jpeg;

procedure TForm1.FormCreate(Sender: TObject);
begin
  glsceneviewer1.buffer.Backgroundcolor:=$845D45;
  GLMaterialLibrary1.TexturePaths:='textures';
  glfreeform1.loadfromfile('acropolis3.bsp');

  { For collision detection }
  glFreeForm1.BuildOctree;

  { Load my textures - not BSP related }
  glmateriallibrary1.Materials.GetLibMaterialByName('clouds').Material.Texture.Image.LoadFromFile('textures/clouds.jpg');
  glmateriallibrary1.Materials.GetLibMaterialByName('sea').Material.Texture.Image.LoadFromFile('textures/sea.jpg');
  glmateriallibrary1.Materials.GetLibMaterialByName('zeus').Material.Texture.Image.LoadFromFile('textures/zeus.tga');

  { Rescale some of the bsp textures to make look nicer }
  glmateriallibrary1.Materials.GetLibMaterialByName('textures/column').TextureScale.X:=2;
  glmateriallibrary1.Materials.GetLibMaterialByName('textures/column').TextureScale.y:=2;
  glmateriallibrary1.Materials.GetLibMaterialByName('textures/column').TextureScale.z:=2;
  glmateriallibrary1.Materials.GetLibMaterialByName('textures/step').TextureScale.X:=2;
  glmateriallibrary1.Materials.GetLibMaterialByName('textures/step').TextureScale.y:=0.02;
  glmateriallibrary1.Materials.GetLibMaterialByName('textures/step').TextureScale.z:=0.02;

  { Set zeus's position }
  glsprite1.position.x:=100;
  glsprite1.position.y:=-110;
  glsprite1.position.z:=37;
end;



//
// events that follow handle camera movements and FPS rate
//

procedure TForm1.Timer1Timer(Sender: TObject);
begin
   Caption:=Format('%.1f FPS', [GLSceneViewer1.FramesPerSecond]);
   GLSceneViewer1.ResetPerformanceMonitor;
end;

procedure TForm1.GLCadencer1Progress(Sender: TObject; const deltaTime,
  newTime: Double);
begin
  GLSceneViewer1.Invalidate;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  factor:double;
  rayStart, rayVector, iPoint, iNormal,oldpos : TVector;
  bFound:boolean;
  hgt:single;

begin
  oldpos:=glcamera1.position.asVector;
  if (ssshift in shift) then
    factor:=4
  else
    factor:=2;
  case key of
    35,40: glcamera1.move(-10*factor);
    37: begin
        glcamera1.turn(-5*factor);

      end;
    39: begin
        glcamera1.turn(5*factor);

      end;
    38,65,16: glcamera1.move(10*factor);
    33:  glcamera1.pitch(1*factor);
    34:  glcamera1.pitch(-1*factor);

  end;

  { Collision detection - use octreeraycastintersect }
  bFound:=false;
  hgt:=glcamera1.position.z;
  while (not bFound) and  (hgt>-100)do
  begin
    iPoint:=GLCamera1.AbsolutePosition;
    iPoint[2]:=hgt;
    SetVector(rayStart, ipoint);
    SetVector(rayVector,  glcamera1.AbsoluteUp   );

    NormalizeVector(rayVector);
    if glFreeForm1.OctreeRayCastIntersect(raystart, rayvector, @iPoint, @iNormal) then
    begin
      hgt:=iPoint[2];
      bFound:=true;

    end
    else
      hgt:=hgt-0.1;
  end;
  if hgt<-100 then
  begin
    glcamera1.Position.asVector:=oldpos;
    caption:='Watch your step';
  end
  else
    glcamera1.Position.z:=hgt+3;
end;

end.

